==> inca_tool-1.0.0-py3-none-any.whl: inca/common/suts.py <==
# Copyright (c) 2022 European Union.
#
# The tool was developed with the contribution of the Joint Research Centre of the European Commission.
#
# This program is free software: you can redistribute it and/or modify it under the terms of the European Union Public
# Licence, either version 1.2 of the License, or (at your option) any later version.
# You may not use this work except in compliance with the Licence.
#
# You may obtain a copy of the Licence at: https://joinup.ec.europa.eu/collection/eupl/eupl-guidelines-faq-infographics
#
# Unless required by applicable law or agreed to in writing, software distributed under the Licence is distributed on
# an "AS IS" basis, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the Licence for the specific language governing permissions and limitations under the Licence.

"""Create supply and use tables in .xlsx format."""

from typing import Union

import openpyxl
import pandas as pd
from openpyxl.styles.alignment import Alignment
from openpyxl.styles.borders import Border, Side
from openpyxl.styles import Font
from openpyxl.utils import get_column_letter
from pandas.api.types import is_integer_dtype

import inca.common.ecosystem as ecosystem

IND = 'Intermediate consumption by industries'
GOV = 'Government final consumption'
HSH = 'Households final consumption'
GCF = 'Gross capital formation'
EXP = 'Exports'

SECTOR = 'Sector'

#: Economic sectors for supply tables.
use_sector_cat = pd.CategoricalDtype([IND, GOV, HSH, GCF, EXP])

df_sectors = pd.DataFrame([[IND, 'P2'],
                           [GOV, 'P3_S13'],
                           [HSH, 'P3_S14'],
                           [GCF, 'P5'],
                           [EXP, 'P6']],
                          columns=[SECTOR, 'short_name']).astype({SECTOR: use_sector_cat})

_NUMBER_FORMAT_FLOAT = '0.00;-0.00;-;@'
_NUMBER_FORMAT_INT = '0;-0;-;@'


def write_sut(filename, supply: Union[pd.DataFrame, pd.Series], use: Union[pd.DataFrame, pd.Series],
              service_name='N/A', year='N/A', unit='N/A'):
    """Write a formatted Excel spreadsheet with supply and use tables.

    :param filename: output file name
    :param use: use table, column labels should be economic sectors from :data:`inca.common.suts.use_sector_cat`
    :param supply: supply table, column labels should be L1 ecosystem type names as defined in
     :data:`inca.common.ecosystem.types_l1`.
    """
    wb = openpyxl.Workbook()
    sheet_use = wb.active
    sheet_use.title = 'Use'
    sheet_supply = wb.create_sheet('Supply')

    if isinstance(use, pd.Series):
        use = use.to_frame()
    if isinstance(supply, pd.Series):
        supply = supply.to_frame()

    assert (all(col_name in df_sectors[SECTOR].values for col_name in use)), \
        'Use table column labels should match standard sector names.'
    assert (all(col_name in ecosystem.types_l1['name'].values for col_name in supply)), \
        'Supply table column labels should match standard ecosystem type names.'

    title = f'{service_name} ({year}) [{unit}]'
    _write_table(sheet_use, use, df_sectors[SECTOR], title)
    _write_table(sheet_supply, supply, ecosystem.types_l1['name'], title)

    wb.save(filename)
    wb.close()


def _write_table(sheet, data, col_labels, title):
    """Write data to a worksheet, and add some formatting."""
    col_regions = 2
    row_title = 2
    row_header = row_title + 1
    row_total = row_header + 1
    col_total = col_regions + 1
    row_data = row_header + 2
    col_data = col_regions + 2

    # write regions
    sheet.cell(row_header + 1, col_regions, value='all regions')
    for row, region in enumerate(data.index, row_data):
        cell = sheet.cell(row, col_regions)
        cell.value = region
    sheet.column_dimensions[get_column_letter(col_regions)].bestFit = True

    # write column labels
    cell = sheet.cell(row_header, col_regions + 1, value='total')
    cell.alignment = Alignment(textRotation=90)
    sheet.column_dimensions[cell.column_letter].bestFit = True
    all_integer = True
    for col, col_name in enumerate(col_labels, col_data):
        # check dtype for this column
        column_is_int = True
        if col_name in data and not is_integer_dtype(data[col_name].dtype):
            column_is_int = False
            all_integer = False
        column_number_format = _NUMBER_FORMAT_INT if column_is_int else _NUMBER_FORMAT_FLOAT

        # create headers for economic sectors
        cell_sector = sheet.cell(row_header, col)
        cell_sector.value = col_name
        cell_sector.alignment = Alignment(textRotation=90)
        sheet.column_dimensions[cell.column_letter].bestFit = True

        # write 'SUM formula in totals row
        top_cell = sheet.cell(row_data, col)
        bottom_cell = sheet.cell(row_data + len(data) - 1, col)
        cell = sheet.cell(row_total, col)
        cell.value = f'=SUM({top_cell.coordinate}:{bottom_cell.coordinate})'
        cell.number_format = column_number_format

        # write data if we have dataframe column for this sector
        if col_name in data:
            for row, value in enumerate(data[col_name].values, row_data):
                cell = sheet.cell(row, col)
                cell.value = value
                cell.number_format = column_number_format
        else:
            for row in range(row_data, row_data + len(data)):
                cell = sheet.cell(row, col)
                cell.value = 0.
                cell.number_format = column_number_format

    # Add 'SUM' formula in totals column:
    for row in range(row_total, row_total + len(data) + 1):
        left_cell = sheet.cell(row, col_data)
        right_cell = sheet.cell(row, col_data + len(col_labels) - 1)
        cell = sheet.cell(row, col_total)
        cell.value = f'=SUM({left_cell.coordinate}:{right_cell.coordinate})'
        cell.number_format = _NUMBER_FORMAT_INT if all_integer else _NUMBER_FORMAT_FLOAT

    # Borders, font:
    thin = Side(style='thin')
    thick = Side(style='thick')
    bold = Font(bold=True)
    for col in range(col_regions, col_regions + 2 + len(col_labels)):
        sheet.cell(row_header, col).border = Border(top=thick)
        sheet.cell(row_header + 1 + len(data), col).border = Border(bottom=thick)
        cell = sheet.cell(row_total, col)
        cell.border = Border(top=thin, bottom=thin)
        cell.font = bold
    for row in range(row_header, row_header + 2 + len(data)):
        cell = sheet.cell(row, col_regions)
        cell.border = Border(left=thick, bottom=cell.border.bottom, top=cell.border.top)
        cell = sheet.cell(row, col_regions + 1 + len(col_labels))
        cell.border = Border(right=thick, bottom=cell.border.bottom, top=cell.border.top)
        cell = sheet.cell(row, col_total)
        cell.border = Border(left=thin, right=thin,
                             bottom=cell.border.bottom, top=cell.border.top)
        cell.font = bold

    # Title:
    sheet.merge_cells(start_row=row_title, end_row=row_title,
                      start_column=col_regions, end_column=col_regions + len(col_labels))
    sheet.cell(row_title, col_regions, value=title)

==> inca_tool-1.1.2-py3-none-any.whl: inca/common/suts.py <==
# Copyright (c) 2022 European Union.
#
# The tool was developed with the contribution of the Joint Research Centre of the European Commission.
#
# This program is free software: you can redistribute it and/or modify it under the terms of the European Union Public
# Licence, either version 1.2 of the License, or (at your option) any later version.
# You may not use this work except in compliance with the Licence.
#
# You may obtain a copy of the Licence at: https://joinup.ec.europa.eu/collection/eupl/eupl-guidelines-faq-infographics
#
# Unless required by applicable law or agreed to in writing, software distributed under the Licence is distributed on
# an "AS IS" basis, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the Licence for the specific language governing permissions and limitations under the Licence.

"""Create supply and use tables in .xlsx format."""

from typing import Union

import openpyxl
import pandas as pd
from openpyxl.styles.alignment import Alignment
from openpyxl.styles.borders import Border, Side
from openpyxl.styles import Font
from openpyxl.utils import get_column_letter
from pandas.api.types import is_integer_dtype

import inca.common.ecosystem as ecosystem

IND = 'Intermediate consumption by industries'
GOV = 'Government final consumption'
HSH = 'Households final consumption'
GCF = 'Gross capital formation'
EXP = 'Exports'

SECTOR = 'Sector'

#: Economic sectors for supply tables.
use_sector_cat = pd.CategoricalDtype([IND, GOV, HSH, GCF, EXP])

df_sectors = pd.DataFrame([[IND, 'P2'],
                           [GOV, 'P3_S13'],
                           [HSH, 'P3_S14'],
                           [GCF, 'P5'],
                           [EXP, 'P6']],
                          columns=[SECTOR, 'short_name']).astype({SECTOR: use_sector_cat})

_NUMBER_FORMAT_FLOAT = '0.00;-0.00;-;@'
_NUMBER_FORMAT_INT = '0;-0;-;@'


def write_sut(filename, supply: Union[pd.DataFrame, pd.Series], use: Union[pd.DataFrame, pd.Series],
              service_name='N/A', year='N/A', unit='N/A', region='all regions'):
    """Write a formatted Excel spreadsheet with supply and use tables.

    :param filename: output file name
    :param use: use table, column labels should be economic sectors from :data:`inca.common.suts.use_sector_cat`
    :param supply: supply table, column labels should be L1 ecosystem type names as defined in
     :data:`inca.common.ecosystem.types_l1`.
    """
    wb = openpyxl.Workbook()
    sheet_use = wb.active
    sheet_use.title = 'Use'
    sheet_supply = wb.create_sheet('Supply')

    if isinstance(use, pd.Series):
        use = use.to_frame()
    if isinstance(supply, pd.Series):
        supply = supply.to_frame()

    assert (all(col_name in df_sectors[SECTOR].values for col_name in use)), \
        'Use table column labels should match standard sector names.'
    assert (all(col_name in ecosystem.types_l1['name'].values for col_name in supply)), \
        'Supply table column labels should match standard ecosystem type names.'

    title = f'{service_name} ({year}) [{unit}]'
    _write_table(sheet_use, use, df_sectors[SECTOR], title, region=region)
    _write_table(sheet_supply, supply, ecosystem.types_l1['name'], title, region=region)

    wb.save(filename)
    wb.close()


def _write_table(sheet, data, col_labels, title, region='all regions'):
    """Write data to a worksheet, and add some formatting."""
    col_regions = 2
    row_title = 2
    row_header = row_title + 1
    row_total = row_header + 1
    col_total = col_regions + 1
    row_data = row_header + 2
    col_data = col_regions + 2

    # write regions
    sheet.cell(row_header + 1, col_regions, value=region)
    for row, region in enumerate(data.index, row_data):
        cell = sheet.cell(row, col_regions)
        cell.value = region
    sheet.column_dimensions[get_column_letter(col_regions)].bestFit = True

    # write column labels
    cell = sheet.cell(row_header, col_regions + 1, value='total')
    cell.alignment = Alignment(textRotation=90)
    sheet.column_dimensions[cell.column_letter].bestFit = True
    all_integer = True
    for col, col_name in enumerate(col_labels, col_data):
        # check dtype for this column
        column_is_int = True
        if col_name in data and not is_integer_dtype(data[col_name].dtype):
            column_is_int = False
            all_integer = False
        column_number_format = _NUMBER_FORMAT_INT if column_is_int else _NUMBER_FORMAT_FLOAT

        # create headers for economic sectors
        cell_sector = sheet.cell(row_header, col)
        cell_sector.value = col_name
        cell_sector.alignment = Alignment(textRotation=90)
        sheet.column_dimensions[cell.column_letter].bestFit = True

        # write 'SUM formula in totals row
        top_cell = sheet.cell(row_data, col)
        bottom_cell = sheet.cell(row_data + len(data) - 1, col)
        cell = sheet.cell(row_total, col)
        cell.value = f'=SUM({top_cell.coordinate}:{bottom_cell.coordinate})'
        cell.number_format = column_number_format

        # write data if we have dataframe column for this sector
        if col_name in data:
            for row, value in enumerate(data[col_name].values, row_data):
                cell = sheet.cell(row, col)
                cell.value = value
                cell.number_format = column_number_format
        else:
            for row in range(row_data, row_data + len(data)):
                cell = sheet.cell(row, col)
                cell.value = 0.
                cell.number_format = column_number_format

    # Add 'SUM' formula in totals column:
    for row in range(row_total, row_total + len(data) + 1):
        left_cell = sheet.cell(row, col_data)
        right_cell = sheet.cell(row, col_data + len(col_labels) - 1)
        cell = sheet.cell(row, col_total)
        cell.value = f'=SUM({left_cell.coordinate}:{right_cell.coordinate})'
        cell.number_format = _NUMBER_FORMAT_INT if all_integer else _NUMBER_FORMAT_FLOAT

    # Borders, font:
    thin = Side(style='thin')
    thick = Side(style='thick')
    bold = Font(bold=True)
    for col in range(col_regions, col_regions + 2 + len(col_labels)):
        sheet.cell(row_header, col).border = Border(top=thick)
        sheet.cell(row_header + 1 + len(data), col).border = Border(bottom=thick)
        cell = sheet.cell(row_total, col)
        cell.border = Border(top=thin, bottom=thin)
        cell.font = bold
    for row in range(row_header, row_header + 2 + len(data)):
        cell = sheet.cell(row, col_regions)
        cell.border = Border(left=thick, bottom=cell.border.bottom, top=cell.border.top)
        cell = sheet.cell(row, col_regions + 1 + len(col_labels))
        cell.border = Border(right=thick, bottom=cell.border.bottom, top=cell.border.top)
        cell = sheet.cell(row, col_total)
        cell.border = Border(left=thin, right=thin,
                             bottom=cell.border.bottom, top=cell.border.top)
        cell.font = bold

    # Title:
    sheet.merge_cells(start_row=row_title, end_row=row_title,
                      start_column=col_regions, end_column=col_regions + len(col_labels))
    sheet.cell(row_title, col_regions, value=title)
